# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import encode_path_param
from ..core.pagination import AsyncPager, SyncPager
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.conflict_error import ConflictError
from ..errors.forbidden_error import ForbiddenError
from ..errors.internal_server_error import InternalServerError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.create_self_service_profile_response_content import CreateSelfServiceProfileResponseContent
from ..types.get_self_service_profile_response_content import GetSelfServiceProfileResponseContent
from ..types.list_self_service_profiles_paginated_response_content import (
    ListSelfServiceProfilesPaginatedResponseContent,
)
from ..types.self_service_profile import SelfServiceProfile
from ..types.self_service_profile_allowed_strategy_enum import SelfServiceProfileAllowedStrategyEnum
from ..types.self_service_profile_branding import SelfServiceProfileBranding
from ..types.self_service_profile_branding_properties import SelfServiceProfileBrandingProperties
from ..types.self_service_profile_description import SelfServiceProfileDescription
from ..types.self_service_profile_user_attribute import SelfServiceProfileUserAttribute
from ..types.self_service_profile_user_attributes import SelfServiceProfileUserAttributes
from ..types.update_self_service_profile_response_content import UpdateSelfServiceProfileResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSelfServiceProfilesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[SelfServiceProfile, ListSelfServiceProfilesPaginatedResponseContent]:
        """
        Retrieves self-service profiles.

        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[SelfServiceProfile, ListSelfServiceProfilesPaginatedResponseContent]
            List of existing profiles.
        """
        page = page if page is not None else 0

        _response = self._client_wrapper.httpx_client.request(
            "self-service-profiles",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
                "include_totals": include_totals,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListSelfServiceProfilesPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListSelfServiceProfilesPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.self_service_profiles
                _has_next = len(_items or []) > 0
                _get_next = lambda: self.list(
                    page=page + 1,
                    per_page=per_page,
                    include_totals=include_totals,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        name: str,
        description: typing.Optional[str] = OMIT,
        branding: typing.Optional[SelfServiceProfileBrandingProperties] = OMIT,
        allowed_strategies: typing.Optional[typing.Sequence[SelfServiceProfileAllowedStrategyEnum]] = OMIT,
        user_attributes: typing.Optional[typing.Sequence[SelfServiceProfileUserAttribute]] = OMIT,
        user_attribute_profile_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateSelfServiceProfileResponseContent]:
        """
        Creates a self-service profile.

        Parameters
        ----------
        name : str
            The name of the self-service Profile.

        description : typing.Optional[str]
            The description of the self-service Profile.

        branding : typing.Optional[SelfServiceProfileBrandingProperties]

        allowed_strategies : typing.Optional[typing.Sequence[SelfServiceProfileAllowedStrategyEnum]]
            List of IdP strategies that will be shown to users during the Self-Service Enterprise Configuration flow. Possible values: [`oidc`, `samlp`, `waad`, `google-apps`, `adfs`, `okta`, `auth0-samlp`, `okta-samlp`, `keycloak-samlp`, `pingfederate`]

        user_attributes : typing.Optional[typing.Sequence[SelfServiceProfileUserAttribute]]
            List of attributes to be mapped that will be shown to the user during the Self-Service Enterprise Configuration flow.

        user_attribute_profile_id : typing.Optional[str]
            ID of the user-attribute-profile to associate with this self-service profile.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateSelfServiceProfileResponseContent]
            Self-service profile successfully created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "self-service-profiles",
            method="POST",
            json={
                "name": name,
                "description": description,
                "branding": convert_and_respect_annotation_metadata(
                    object_=branding, annotation=SelfServiceProfileBrandingProperties, direction="write"
                ),
                "allowed_strategies": allowed_strategies,
                "user_attributes": convert_and_respect_annotation_metadata(
                    object_=user_attributes,
                    annotation=typing.Sequence[SelfServiceProfileUserAttribute],
                    direction="write",
                ),
                "user_attribute_profile_id": user_attribute_profile_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateSelfServiceProfileResponseContent,
                    parse_obj_as(
                        type_=CreateSelfServiceProfileResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetSelfServiceProfileResponseContent]:
        """
        Retrieves a self-service profile by Id.

        Parameters
        ----------
        id : str
            The id of the self-service profile to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetSelfServiceProfileResponseContent]
            Record for existing self-service profile.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"self-service-profiles/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSelfServiceProfileResponseContent,
                    parse_obj_as(
                        type_=GetSelfServiceProfileResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Deletes a self-service profile by Id.

        Parameters
        ----------
        id : str
            The id of the self-service profile to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"self-service-profiles/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[SelfServiceProfileDescription] = OMIT,
        branding: typing.Optional[SelfServiceProfileBranding] = OMIT,
        allowed_strategies: typing.Optional[typing.Sequence[SelfServiceProfileAllowedStrategyEnum]] = OMIT,
        user_attributes: typing.Optional[SelfServiceProfileUserAttributes] = OMIT,
        user_attribute_profile_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateSelfServiceProfileResponseContent]:
        """
        Updates a self-service profile.

        Parameters
        ----------
        id : str
            The id of the self-service profile to update

        name : typing.Optional[str]
            The name of the self-service Profile.

        description : typing.Optional[SelfServiceProfileDescription]

        branding : typing.Optional[SelfServiceProfileBranding]

        allowed_strategies : typing.Optional[typing.Sequence[SelfServiceProfileAllowedStrategyEnum]]
            List of IdP strategies that will be shown to users during the Self-Service Enterprise Configuration flow. Possible values: [`oidc`, `samlp`, `waad`, `google-apps`, `adfs`, `okta`, `auth0-samlp`, `okta-samlp`, `keycloak-samlp`, `pingfederate`]

        user_attributes : typing.Optional[SelfServiceProfileUserAttributes]

        user_attribute_profile_id : typing.Optional[str]
            ID of the user-attribute-profile to associate with this self-service profile.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateSelfServiceProfileResponseContent]
            Self-service profile successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"self-service-profiles/{encode_path_param(id)}",
            method="PATCH",
            json={
                "name": name,
                "description": description,
                "branding": convert_and_respect_annotation_metadata(
                    object_=branding, annotation=SelfServiceProfileBranding, direction="write"
                ),
                "allowed_strategies": allowed_strategies,
                "user_attributes": convert_and_respect_annotation_metadata(
                    object_=user_attributes,
                    annotation=typing.Optional[SelfServiceProfileUserAttributes],
                    direction="write",
                ),
                "user_attribute_profile_id": user_attribute_profile_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateSelfServiceProfileResponseContent,
                    parse_obj_as(
                        type_=UpdateSelfServiceProfileResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSelfServiceProfilesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[SelfServiceProfile, ListSelfServiceProfilesPaginatedResponseContent]:
        """
        Retrieves self-service profiles.

        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[SelfServiceProfile, ListSelfServiceProfilesPaginatedResponseContent]
            List of existing profiles.
        """
        page = page if page is not None else 0

        _response = await self._client_wrapper.httpx_client.request(
            "self-service-profiles",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
                "include_totals": include_totals,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListSelfServiceProfilesPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListSelfServiceProfilesPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.self_service_profiles
                _has_next = len(_items or []) > 0

                async def _get_next():
                    return await self.list(
                        page=page + 1,
                        per_page=per_page,
                        include_totals=include_totals,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        name: str,
        description: typing.Optional[str] = OMIT,
        branding: typing.Optional[SelfServiceProfileBrandingProperties] = OMIT,
        allowed_strategies: typing.Optional[typing.Sequence[SelfServiceProfileAllowedStrategyEnum]] = OMIT,
        user_attributes: typing.Optional[typing.Sequence[SelfServiceProfileUserAttribute]] = OMIT,
        user_attribute_profile_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateSelfServiceProfileResponseContent]:
        """
        Creates a self-service profile.

        Parameters
        ----------
        name : str
            The name of the self-service Profile.

        description : typing.Optional[str]
            The description of the self-service Profile.

        branding : typing.Optional[SelfServiceProfileBrandingProperties]

        allowed_strategies : typing.Optional[typing.Sequence[SelfServiceProfileAllowedStrategyEnum]]
            List of IdP strategies that will be shown to users during the Self-Service Enterprise Configuration flow. Possible values: [`oidc`, `samlp`, `waad`, `google-apps`, `adfs`, `okta`, `auth0-samlp`, `okta-samlp`, `keycloak-samlp`, `pingfederate`]

        user_attributes : typing.Optional[typing.Sequence[SelfServiceProfileUserAttribute]]
            List of attributes to be mapped that will be shown to the user during the Self-Service Enterprise Configuration flow.

        user_attribute_profile_id : typing.Optional[str]
            ID of the user-attribute-profile to associate with this self-service profile.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateSelfServiceProfileResponseContent]
            Self-service profile successfully created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "self-service-profiles",
            method="POST",
            json={
                "name": name,
                "description": description,
                "branding": convert_and_respect_annotation_metadata(
                    object_=branding, annotation=SelfServiceProfileBrandingProperties, direction="write"
                ),
                "allowed_strategies": allowed_strategies,
                "user_attributes": convert_and_respect_annotation_metadata(
                    object_=user_attributes,
                    annotation=typing.Sequence[SelfServiceProfileUserAttribute],
                    direction="write",
                ),
                "user_attribute_profile_id": user_attribute_profile_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateSelfServiceProfileResponseContent,
                    parse_obj_as(
                        type_=CreateSelfServiceProfileResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetSelfServiceProfileResponseContent]:
        """
        Retrieves a self-service profile by Id.

        Parameters
        ----------
        id : str
            The id of the self-service profile to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetSelfServiceProfileResponseContent]
            Record for existing self-service profile.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"self-service-profiles/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSelfServiceProfileResponseContent,
                    parse_obj_as(
                        type_=GetSelfServiceProfileResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Deletes a self-service profile by Id.

        Parameters
        ----------
        id : str
            The id of the self-service profile to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"self-service-profiles/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[SelfServiceProfileDescription] = OMIT,
        branding: typing.Optional[SelfServiceProfileBranding] = OMIT,
        allowed_strategies: typing.Optional[typing.Sequence[SelfServiceProfileAllowedStrategyEnum]] = OMIT,
        user_attributes: typing.Optional[SelfServiceProfileUserAttributes] = OMIT,
        user_attribute_profile_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateSelfServiceProfileResponseContent]:
        """
        Updates a self-service profile.

        Parameters
        ----------
        id : str
            The id of the self-service profile to update

        name : typing.Optional[str]
            The name of the self-service Profile.

        description : typing.Optional[SelfServiceProfileDescription]

        branding : typing.Optional[SelfServiceProfileBranding]

        allowed_strategies : typing.Optional[typing.Sequence[SelfServiceProfileAllowedStrategyEnum]]
            List of IdP strategies that will be shown to users during the Self-Service Enterprise Configuration flow. Possible values: [`oidc`, `samlp`, `waad`, `google-apps`, `adfs`, `okta`, `auth0-samlp`, `okta-samlp`, `keycloak-samlp`, `pingfederate`]

        user_attributes : typing.Optional[SelfServiceProfileUserAttributes]

        user_attribute_profile_id : typing.Optional[str]
            ID of the user-attribute-profile to associate with this self-service profile.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateSelfServiceProfileResponseContent]
            Self-service profile successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"self-service-profiles/{encode_path_param(id)}",
            method="PATCH",
            json={
                "name": name,
                "description": description,
                "branding": convert_and_respect_annotation_metadata(
                    object_=branding, annotation=SelfServiceProfileBranding, direction="write"
                ),
                "allowed_strategies": allowed_strategies,
                "user_attributes": convert_and_respect_annotation_metadata(
                    object_=user_attributes,
                    annotation=typing.Optional[SelfServiceProfileUserAttributes],
                    direction="write",
                ),
                "user_attribute_profile_id": user_attribute_profile_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateSelfServiceProfileResponseContent,
                    parse_obj_as(
                        type_=UpdateSelfServiceProfileResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.create_scim_configuration_request_content import CreateScimConfigurationRequestContent
from ...types.create_scim_configuration_response_content import CreateScimConfigurationResponseContent
from ...types.get_scim_configuration_default_mapping_response_content import (
    GetScimConfigurationDefaultMappingResponseContent,
)
from ...types.get_scim_configuration_response_content import GetScimConfigurationResponseContent
from ...types.list_scim_configurations_response_content import ListScimConfigurationsResponseContent
from ...types.scim_configuration import ScimConfiguration
from ...types.scim_mapping_item import ScimMappingItem
from ...types.update_scim_configuration_response_content import UpdateScimConfigurationResponseContent
from .raw_client import AsyncRawScimConfigurationClient, RawScimConfigurationClient

if typing.TYPE_CHECKING:
    from .tokens.client import AsyncTokensClient, TokensClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ScimConfigurationClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawScimConfigurationClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._tokens: typing.Optional[TokensClient] = None

    @property
    def with_raw_response(self) -> RawScimConfigurationClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawScimConfigurationClient
        """
        return self._raw_client

    def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ScimConfiguration, ListScimConfigurationsResponseContent]:
        """
        Retrieve a list of SCIM configurations of a tenant.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ScimConfiguration, ListScimConfigurationsResponseContent]
            The tenant's SCIM configurations. See <strong>Response Schema</strong> for schema.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.connections.scim_configuration.list(
            from_="from",
            take=1,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(from_=from_, take=take, request_options=request_options)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetScimConfigurationResponseContent:
        """
        Retrieves a scim configuration by its `connectionId`.

        Parameters
        ----------
        id : str
            The id of the connection to retrieve its SCIM configuration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetScimConfigurationResponseContent
            The connection's SCIM configuration was retrieved. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.scim_configuration.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def create(
        self,
        id: str,
        *,
        request: typing.Optional[CreateScimConfigurationRequestContent] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateScimConfigurationResponseContent:
        """
        Create a scim configuration for a connection.

        Parameters
        ----------
        id : str
            The id of the connection to create its SCIM configuration

        request : typing.Optional[CreateScimConfigurationRequestContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateScimConfigurationResponseContent
            The connection's SCIM configuration was created. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        from auth0 import Auth0, CreateScimConfigurationRequestContent

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.scim_configuration.create(
            id="id",
            request=CreateScimConfigurationRequestContent(),
        )
        """
        _response = self._raw_client.create(id, request=request, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Deletes a scim configuration by its `connectionId`.

        Parameters
        ----------
        id : str
            The id of the connection to delete its SCIM configuration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.scim_configuration.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        user_id_attribute: str,
        mapping: typing.Sequence[ScimMappingItem],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateScimConfigurationResponseContent:
        """
        Update a scim configuration by its `connectionId`.

        Parameters
        ----------
        id : str
            The id of the connection to update its SCIM configuration

        user_id_attribute : str
            User ID attribute for generating unique user ids

        mapping : typing.Sequence[ScimMappingItem]
            The mapping between auth0 and SCIM

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateScimConfigurationResponseContent
            The connection's SCIM configuration was updated. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        from auth0 import Auth0, ScimMappingItem

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.scim_configuration.update(
            id="id",
            user_id_attribute="user_id_attribute",
            mapping=[ScimMappingItem()],
        )
        """
        _response = self._raw_client.update(
            id, user_id_attribute=user_id_attribute, mapping=mapping, request_options=request_options
        )
        return _response.data

    def get_default_mapping(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetScimConfigurationDefaultMappingResponseContent:
        """
        Retrieves a scim configuration's default mapping by its `connectionId`.

        Parameters
        ----------
        id : str
            The id of the connection to retrieve its default SCIM mapping

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetScimConfigurationDefaultMappingResponseContent
            The connection's default SCIM mapping was retrieved. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.connections.scim_configuration.get_default_mapping(
            id="id",
        )
        """
        _response = self._raw_client.get_default_mapping(id, request_options=request_options)
        return _response.data

    @property
    def tokens(self):
        if self._tokens is None:
            from .tokens.client import TokensClient  # noqa: E402

            self._tokens = TokensClient(client_wrapper=self._client_wrapper)
        return self._tokens


class AsyncScimConfigurationClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawScimConfigurationClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._tokens: typing.Optional[AsyncTokensClient] = None

    @property
    def with_raw_response(self) -> AsyncRawScimConfigurationClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawScimConfigurationClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ScimConfiguration, ListScimConfigurationsResponseContent]:
        """
        Retrieve a list of SCIM configurations of a tenant.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ScimConfiguration, ListScimConfigurationsResponseContent]
            The tenant's SCIM configurations. See <strong>Response Schema</strong> for schema.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.connections.scim_configuration.list(
                from_="from",
                take=1,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(from_=from_, take=take, request_options=request_options)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetScimConfigurationResponseContent:
        """
        Retrieves a scim configuration by its `connectionId`.

        Parameters
        ----------
        id : str
            The id of the connection to retrieve its SCIM configuration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetScimConfigurationResponseContent
            The connection's SCIM configuration was retrieved. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.scim_configuration.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def create(
        self,
        id: str,
        *,
        request: typing.Optional[CreateScimConfigurationRequestContent] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateScimConfigurationResponseContent:
        """
        Create a scim configuration for a connection.

        Parameters
        ----------
        id : str
            The id of the connection to create its SCIM configuration

        request : typing.Optional[CreateScimConfigurationRequestContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateScimConfigurationResponseContent
            The connection's SCIM configuration was created. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0, CreateScimConfigurationRequestContent

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.scim_configuration.create(
                id="id",
                request=CreateScimConfigurationRequestContent(),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(id, request=request, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Deletes a scim configuration by its `connectionId`.

        Parameters
        ----------
        id : str
            The id of the connection to delete its SCIM configuration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.scim_configuration.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        user_id_attribute: str,
        mapping: typing.Sequence[ScimMappingItem],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateScimConfigurationResponseContent:
        """
        Update a scim configuration by its `connectionId`.

        Parameters
        ----------
        id : str
            The id of the connection to update its SCIM configuration

        user_id_attribute : str
            User ID attribute for generating unique user ids

        mapping : typing.Sequence[ScimMappingItem]
            The mapping between auth0 and SCIM

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateScimConfigurationResponseContent
            The connection's SCIM configuration was updated. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0, ScimMappingItem

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.scim_configuration.update(
                id="id",
                user_id_attribute="user_id_attribute",
                mapping=[ScimMappingItem()],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id, user_id_attribute=user_id_attribute, mapping=mapping, request_options=request_options
        )
        return _response.data

    async def get_default_mapping(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetScimConfigurationDefaultMappingResponseContent:
        """
        Retrieves a scim configuration's default mapping by its `connectionId`.

        Parameters
        ----------
        id : str
            The id of the connection to retrieve its default SCIM mapping

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetScimConfigurationDefaultMappingResponseContent
            The connection's default SCIM mapping was retrieved. See <strong>Response Schemas</strong> for schema.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.connections.scim_configuration.get_default_mapping(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_default_mapping(id, request_options=request_options)
        return _response.data

    @property
    def tokens(self):
        if self._tokens is None:
            from .tokens.client import AsyncTokensClient  # noqa: E402

            self._tokens = AsyncTokensClient(client_wrapper=self._client_wrapper)
        return self._tokens



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .connection_allowed_audiences_google_o_auth_2 import ConnectionAllowedAudiencesGoogleOAuth2
from .connection_client_id_google_o_auth_2 import ConnectionClientIdGoogleOAuth2
from .connection_client_secret_google_o_auth_2 import ConnectionClientSecretGoogleOAuth2
from .connection_freeform_scopes_google_o_auth_2 import ConnectionFreeformScopesGoogleOAuth2
from .connection_icon_url_google_o_auth_2 import ConnectionIconUrlGoogleOAuth2
from .connection_options_common import ConnectionOptionsCommon
from .connection_scope_google_o_auth_2 import ConnectionScopeGoogleOAuth2
from .connection_set_user_root_attributes_enum import ConnectionSetUserRootAttributesEnum
from .connection_upstream_params import ConnectionUpstreamParams


class ConnectionOptionsGoogleOAuth2(ConnectionOptionsCommon):
    """
    Options for the 'google-oauth2' connection
    """

    allowed_audiences: typing.Optional[ConnectionAllowedAudiencesGoogleOAuth2] = None
    client_id: typing.Optional[ConnectionClientIdGoogleOAuth2] = None
    client_secret: typing.Optional[ConnectionClientSecretGoogleOAuth2] = None
    freeform_scopes: typing.Optional[ConnectionFreeformScopesGoogleOAuth2] = None
    icon_url: typing.Optional[ConnectionIconUrlGoogleOAuth2] = None
    scope: typing.Optional[ConnectionScopeGoogleOAuth2] = None
    set_user_root_attributes: typing.Optional[ConnectionSetUserRootAttributesEnum] = None
    upstream_params: typing.Optional[ConnectionUpstreamParams] = None
    adsense_management: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View and manage user's ad applications, ad units, and channels in AdSense
    """

    analytics: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View user's configuration information and reports
    """

    blogger: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View and manage user's posts and blogs on Blogger and Blogger comments
    """

    calendar: typing.Optional[bool] = pydantic.Field(default=None)
    """
    See, edit, share, and permanently delete all the calendars you can access using Google Calendar
    """

    calendar_addons_execute: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Run as a Calendar add-on
    """

    calendar_events: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View and edit events on all your calendars
    """

    calendar_events_readonly: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View events on all your calendars
    """

    calendar_settings_readonly: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View your Calendar settings
    """

    chrome_web_store: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Read access to user's chrome web store
    """

    contacts: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Full access to the authenticated user's contacts
    """

    contacts_new: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Full access to the authenticated user's contacts
    """

    contacts_other_readonly: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Read-only access to the authenticated user's 'Other contacts'
    """

    contacts_readonly: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Read-only access to the authenticated user's contacts
    """

    content_api_for_shopping: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View and manage user's products, feeds, and subaccounts
    """

    coordinate: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants read and write access to the Coordinate API
    """

    coordinate_readonly: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Grants read access to the Coordinate API
    """

    directory_readonly: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Read-only access to the authenticated user's corporate directory (if applicable)
    """

    document_list: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Access to Google Docs document list feed
    """

    drive: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Full access to all files and folders in the user's Google Drive
    """

    drive_activity: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View and add to the activity record of files in your Drive
    """

    drive_activity_readonly: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View the activity record of files in your Drive
    """

    drive_appdata: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Access to the application's configuration data in the user's Google Drive
    """

    drive_apps_readonly: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View apps authorized to access your Drive
    """

    drive_file: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Access to files created or opened by the app
    """

    drive_metadata: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Access to file metadata, including listing files and folders
    """

    drive_metadata_readonly: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Read-only access to file metadata
    """

    drive_photos_readonly: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Read-only access to the user's Google Photos
    """

    drive_readonly: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Read-only access to all files and folders in the user's Google Drive
    """

    drive_scripts: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Modify the behavior of Google Apps Scripts
    """

    email: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Email and verified email flag
    """

    gmail: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Full access to the account's mailboxes, including permanent deletion of threads and messages
    """

    gmail_compose: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Read all resources and their metadata—no write operations
    """

    gmail_insert: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Insert and import messages only
    """

    gmail_labels: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Create, read, update, and delete labels only
    """

    gmail_metadata: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Read resources metadata including labels, history records, and email message headers, but not the message body or attachments
    """

    gmail_modify: typing.Optional[bool] = pydantic.Field(default=None)
    """
    All read/write operations except immediate, permanent deletion of threads and messages, bypassing Trash
    """

    gmail_new: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Full access to the account's mailboxes, including permanent deletion of threads and messages
    """

    gmail_readonly: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Read all resources and their metadata—no write operations
    """

    gmail_send: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Send messages only. No read or modify privileges on mailbox
    """

    gmail_settings_basic: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Manage basic mail settings
    """

    gmail_settings_sharing: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Manage sensitive mail settings, including forwarding rules and aliases. Note: Operations guarded by this scope are restricted to administrative use only
    """

    google_affiliate_network: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View and manage user's publisher data in the Google Affiliate Network
    """

    google_books: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View and manage user's books and library in Google Books
    """

    google_cloud_storage: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View and manage user's data stored in Google Cloud Storage
    """

    google_drive: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Full access to all files and folders in the user's Google Drive
    """

    google_drive_files: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Access to files created or opened by the app
    """

    google_plus: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Associate user with its public Google profile
    """

    latitude_best: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View and manage user's best-available current location and location history in Google Latitude
    """

    latitude_city: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View and manage user's city-level current location and location history in Google Latitude
    """

    moderator: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View and manage user's votes, topics, and submissions
    """

    offline_access: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Request a refresh token when the user authorizes your application
    """

    orkut: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View and manage user's friends, applications and profile and status
    """

    picasa_web: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View and manage user's Google photos, videos, photo and video tags and comments
    """

    profile: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Name, public profile URL, photo, country, language, and timezone
    """

    sites: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View and manage user's sites on Google Sites
    """

    tasks: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Full access to create, edit, organize, and delete all your tasks
    """

    tasks_readonly: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Read-only access to view your tasks and task lists
    """

    url_shortener: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View, manage and view statistics user's short URLs
    """

    webmaster_tools: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View and manage user's sites and messages, view keywords
    """

    youtube: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Manage your YouTube account
    """

    youtube_channelmemberships_creator: typing.Optional[bool] = pydantic.Field(default=None)
    """
    See a list of your current active channel members, their current level, and when they became a member
    """

    youtube_new: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Manage your YouTube account
    """

    youtube_readonly: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View your YouTube account
    """

    youtube_upload: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Manage your YouTube videos
    """

    youtubepartner: typing.Optional[bool] = pydantic.Field(default=None)
    """
    View and manage your assets and associated content on YouTube
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_logs_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "logs.list_.0"
    client = get_client(test_id)
    client.logs.list(
        page=1,
        per_page=1,
        sort="sort",
        fields="fields",
        include_fields=True,
        include_totals=True,
        search="search",
    )
    verify_request_count(
        test_id,
        "GET",
        "/logs",
        {
            "page": "1",
            "per_page": "1",
            "sort": "sort",
            "fields": "fields",
            "include_fields": "true",
            "include_totals": "true",
            "search": "search",
        },
        1,
    )


def test_logs_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "logs.get.0"
    client = get_client(test_id)
    client.logs.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/logs/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

from .connection_upstream_additional_properties import ConnectionUpstreamAdditionalProperties

ConnectionUpstreamParams = typing.Optional[typing.Dict[str, typing.Optional[ConnectionUpstreamAdditionalProperties]]]
"""
Options for adding parameters in the request to the upstream IdP
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .user_attribute_profile_saml_mapping import UserAttributeProfileSamlMapping
from .user_attribute_profile_user_id_oidc_strategy_override_mapping import (
    UserAttributeProfileUserIdOidcStrategyOverrideMapping,
)


class UserAttributeProfileStrategyOverridesUserIdMapping(UniversalBaseModel):
    oidc_mapping: typing.Optional[UserAttributeProfileUserIdOidcStrategyOverrideMapping] = None
    saml_mapping: typing.Optional[UserAttributeProfileSamlMapping] = None
    scim_mapping: typing.Optional[str] = pydantic.Field(default=None)
    """
    SCIM mapping override for this strategy
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.jsonable_encoder import encode_path_param
from ....core.parse_error import ParsingError
from ....core.pydantic_utilities import parse_obj_as
from ....core.request_options import RequestOptions
from ....core.serialization import convert_and_respect_annotation_metadata
from ....errors.bad_request_error import BadRequestError
from ....errors.conflict_error import ConflictError
from ....errors.forbidden_error import ForbiddenError
from ....errors.not_found_error import NotFoundError
from ....errors.too_many_requests_error import TooManyRequestsError
from ....errors.unauthorized_error import UnauthorizedError
from ....types.create_phone_template_response_content import CreatePhoneTemplateResponseContent
from ....types.create_phone_template_test_notification_response_content import (
    CreatePhoneTemplateTestNotificationResponseContent,
)
from ....types.get_phone_template_response_content import GetPhoneTemplateResponseContent
from ....types.list_phone_templates_response_content import ListPhoneTemplatesResponseContent
from ....types.partial_phone_template_content import PartialPhoneTemplateContent
from ....types.phone_provider_delivery_method_enum import PhoneProviderDeliveryMethodEnum
from ....types.phone_template_content import PhoneTemplateContent
from ....types.phone_template_notification_type_enum import PhoneTemplateNotificationTypeEnum
from ....types.reset_phone_template_request_content import ResetPhoneTemplateRequestContent
from ....types.reset_phone_template_response_content import ResetPhoneTemplateResponseContent
from ....types.update_phone_template_response_content import UpdatePhoneTemplateResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawTemplatesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, *, disabled: typing.Optional[bool] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ListPhoneTemplatesResponseContent]:
        """
        Parameters
        ----------
        disabled : typing.Optional[bool]
            Whether the template is enabled (false) or disabled (true).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListPhoneTemplatesResponseContent]
            The phone notification templates were retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "branding/phone/templates",
            method="GET",
            params={
                "disabled": disabled,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListPhoneTemplatesResponseContent,
                    parse_obj_as(
                        type_=ListPhoneTemplatesResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        type: typing.Optional[PhoneTemplateNotificationTypeEnum] = OMIT,
        disabled: typing.Optional[bool] = OMIT,
        content: typing.Optional[PhoneTemplateContent] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreatePhoneTemplateResponseContent]:
        """
        Parameters
        ----------
        type : typing.Optional[PhoneTemplateNotificationTypeEnum]

        disabled : typing.Optional[bool]
            Whether the template is enabled (false) or disabled (true).

        content : typing.Optional[PhoneTemplateContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreatePhoneTemplateResponseContent]
            The phone notification template was created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "branding/phone/templates",
            method="POST",
            json={
                "type": type,
                "disabled": disabled,
                "content": convert_and_respect_annotation_metadata(
                    object_=content, annotation=PhoneTemplateContent, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreatePhoneTemplateResponseContent,
                    parse_obj_as(
                        type_=CreatePhoneTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetPhoneTemplateResponseContent]:
        """
        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetPhoneTemplateResponseContent]
            The phone notification template were retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"branding/phone/templates/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetPhoneTemplateResponseContent,
                    parse_obj_as(
                        type_=GetPhoneTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"branding/phone/templates/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        content: typing.Optional[PartialPhoneTemplateContent] = OMIT,
        disabled: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdatePhoneTemplateResponseContent]:
        """
        Parameters
        ----------
        id : str

        content : typing.Optional[PartialPhoneTemplateContent]

        disabled : typing.Optional[bool]
            Whether the template is enabled (false) or disabled (true).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdatePhoneTemplateResponseContent]
            The phone notification template was updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"branding/phone/templates/{encode_path_param(id)}",
            method="PATCH",
            json={
                "content": convert_and_respect_annotation_metadata(
                    object_=content, annotation=PartialPhoneTemplateContent, direction="write"
                ),
                "disabled": disabled,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdatePhoneTemplateResponseContent,
                    parse_obj_as(
                        type_=UpdatePhoneTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def reset(
        self,
        id: str,
        *,
        request: ResetPhoneTemplateRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ResetPhoneTemplateResponseContent]:
        """
        Parameters
        ----------
        id : str

        request : ResetPhoneTemplateRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ResetPhoneTemplateResponseContent]
            The phone notification template was reset.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"branding/phone/templates/{encode_path_param(id)}/reset",
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResetPhoneTemplateResponseContent,
                    parse_obj_as(
                        type_=ResetPhoneTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def test(
        self,
        id: str,
        *,
        to: str,
        delivery_method: typing.Optional[PhoneProviderDeliveryMethodEnum] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreatePhoneTemplateTestNotificationResponseContent]:
        """
        Parameters
        ----------
        id : str

        to : str
            Destination of the testing phone notification

        delivery_method : typing.Optional[PhoneProviderDeliveryMethodEnum]
            Medium to use to send the notification

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreatePhoneTemplateTestNotificationResponseContent]
            The phone testing notification for the template was sent
        """
        _response = self._client_wrapper.httpx_client.request(
            f"branding/phone/templates/{encode_path_param(id)}/try",
            method="POST",
            json={
                "to": to,
                "delivery_method": delivery_method,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreatePhoneTemplateTestNotificationResponseContent,
                    parse_obj_as(
                        type_=CreatePhoneTemplateTestNotificationResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawTemplatesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, *, disabled: typing.Optional[bool] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ListPhoneTemplatesResponseContent]:
        """
        Parameters
        ----------
        disabled : typing.Optional[bool]
            Whether the template is enabled (false) or disabled (true).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListPhoneTemplatesResponseContent]
            The phone notification templates were retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "branding/phone/templates",
            method="GET",
            params={
                "disabled": disabled,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListPhoneTemplatesResponseContent,
                    parse_obj_as(
                        type_=ListPhoneTemplatesResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        type: typing.Optional[PhoneTemplateNotificationTypeEnum] = OMIT,
        disabled: typing.Optional[bool] = OMIT,
        content: typing.Optional[PhoneTemplateContent] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreatePhoneTemplateResponseContent]:
        """
        Parameters
        ----------
        type : typing.Optional[PhoneTemplateNotificationTypeEnum]

        disabled : typing.Optional[bool]
            Whether the template is enabled (false) or disabled (true).

        content : typing.Optional[PhoneTemplateContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreatePhoneTemplateResponseContent]
            The phone notification template was created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "branding/phone/templates",
            method="POST",
            json={
                "type": type,
                "disabled": disabled,
                "content": convert_and_respect_annotation_metadata(
                    object_=content, annotation=PhoneTemplateContent, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreatePhoneTemplateResponseContent,
                    parse_obj_as(
                        type_=CreatePhoneTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetPhoneTemplateResponseContent]:
        """
        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetPhoneTemplateResponseContent]
            The phone notification template were retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"branding/phone/templates/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetPhoneTemplateResponseContent,
                    parse_obj_as(
                        type_=GetPhoneTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Parameters
        ----------
        id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"branding/phone/templates/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        content: typing.Optional[PartialPhoneTemplateContent] = OMIT,
        disabled: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdatePhoneTemplateResponseContent]:
        """
        Parameters
        ----------
        id : str

        content : typing.Optional[PartialPhoneTemplateContent]

        disabled : typing.Optional[bool]
            Whether the template is enabled (false) or disabled (true).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdatePhoneTemplateResponseContent]
            The phone notification template was updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"branding/phone/templates/{encode_path_param(id)}",
            method="PATCH",
            json={
                "content": convert_and_respect_annotation_metadata(
                    object_=content, annotation=PartialPhoneTemplateContent, direction="write"
                ),
                "disabled": disabled,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdatePhoneTemplateResponseContent,
                    parse_obj_as(
                        type_=UpdatePhoneTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def reset(
        self,
        id: str,
        *,
        request: ResetPhoneTemplateRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ResetPhoneTemplateResponseContent]:
        """
        Parameters
        ----------
        id : str

        request : ResetPhoneTemplateRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ResetPhoneTemplateResponseContent]
            The phone notification template was reset.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"branding/phone/templates/{encode_path_param(id)}/reset",
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ResetPhoneTemplateResponseContent,
                    parse_obj_as(
                        type_=ResetPhoneTemplateResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def test(
        self,
        id: str,
        *,
        to: str,
        delivery_method: typing.Optional[PhoneProviderDeliveryMethodEnum] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreatePhoneTemplateTestNotificationResponseContent]:
        """
        Parameters
        ----------
        id : str

        to : str
            Destination of the testing phone notification

        delivery_method : typing.Optional[PhoneProviderDeliveryMethodEnum]
            Medium to use to send the notification

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreatePhoneTemplateTestNotificationResponseContent]
            The phone testing notification for the template was sent
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"branding/phone/templates/{encode_path_param(id)}/try",
            method="POST",
            json={
                "to": to,
                "delivery_method": delivery_method,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreatePhoneTemplateTestNotificationResponseContent,
                    parse_obj_as(
                        type_=CreatePhoneTemplateTestNotificationResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .connection_https_url_with_http_fallback_255 import ConnectionHttpsUrlWithHttpFallback255

ConnectionSignInEndpointAd = ConnectionHttpsUrlWithHttpFallback255
"""
The sign-in endpoint type for the AD-LDAP connector agent (managed by the AD Connector agent).
"""



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.create_hook_response_content import CreateHookResponseContent
from ..types.get_hook_response_content import GetHookResponseContent
from ..types.hook import Hook
from ..types.hook_dependencies import HookDependencies
from ..types.hook_trigger_id_enum import HookTriggerIdEnum
from ..types.list_hooks_offset_paginated_response_content import ListHooksOffsetPaginatedResponseContent
from ..types.update_hook_response_content import UpdateHookResponseContent
from .raw_client import AsyncRawHooksClient, RawHooksClient

if typing.TYPE_CHECKING:
    from .secrets.client import AsyncSecretsClient, SecretsClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class HooksClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawHooksClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._secrets: typing.Optional[SecretsClient] = None

    @property
    def with_raw_response(self) -> RawHooksClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawHooksClient
        """
        return self._raw_client

    def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        enabled: typing.Optional[bool] = None,
        fields: typing.Optional[str] = None,
        trigger_id: typing.Optional[HookTriggerIdEnum] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Hook, ListHooksOffsetPaginatedResponseContent]:
        """
        Retrieve all [hooks](https://auth0.com/docs/hooks). Accepts a list of fields to include or exclude in the result.

        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        enabled : typing.Optional[bool]
            Optional filter on whether a hook is enabled (true) or disabled (false).

        fields : typing.Optional[str]
            Comma-separated list of fields to include in the result. Leave empty to retrieve all fields.

        trigger_id : typing.Optional[HookTriggerIdEnum]
            Retrieves hooks that match the trigger

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Hook, ListHooksOffsetPaginatedResponseContent]
            Hooks successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.hooks.list(
            page=1,
            per_page=1,
            include_totals=True,
            enabled=True,
            fields="fields",
            trigger_id="credentials-exchange",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            page=page,
            per_page=per_page,
            include_totals=include_totals,
            enabled=enabled,
            fields=fields,
            trigger_id=trigger_id,
            request_options=request_options,
        )

    def create(
        self,
        *,
        name: str,
        script: str,
        trigger_id: HookTriggerIdEnum,
        enabled: typing.Optional[bool] = OMIT,
        dependencies: typing.Optional[HookDependencies] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateHookResponseContent:
        """
        Create a new hook.

        Parameters
        ----------
        name : str
            Name of this hook.

        script : str
            Code to be executed when this hook runs.

        trigger_id : HookTriggerIdEnum
            Execution stage of this rule. Can be `credentials-exchange`, `pre-user-registration`, `post-user-registration`, `post-change-password`, or `send-phone-message`.

        enabled : typing.Optional[bool]
            Whether this hook will be executed (true) or ignored (false).

        dependencies : typing.Optional[HookDependencies]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateHookResponseContent
            Hook successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.hooks.create(
            name="name",
            script="script",
            trigger_id="credentials-exchange",
        )
        """
        _response = self._raw_client.create(
            name=name,
            script=script,
            trigger_id=trigger_id,
            enabled=enabled,
            dependencies=dependencies,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, id: str, *, fields: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> GetHookResponseContent:
        """
        Retrieve [a hook](https://auth0.com/docs/hooks) by its ID. Accepts a list of fields to include in the result.

        Parameters
        ----------
        id : str
            ID of the hook to retrieve.

        fields : typing.Optional[str]
            Comma-separated list of fields to include in the result. Leave empty to retrieve all fields.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetHookResponseContent
            Hook successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.hooks.get(
            id="id",
            fields="fields",
        )
        """
        _response = self._raw_client.get(id, fields=fields, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a hook.

        Parameters
        ----------
        id : str
            ID of the hook to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.hooks.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        script: typing.Optional[str] = OMIT,
        enabled: typing.Optional[bool] = OMIT,
        dependencies: typing.Optional[HookDependencies] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateHookResponseContent:
        """
        Update an existing hook.

        Parameters
        ----------
        id : str
            ID of the hook to update.

        name : typing.Optional[str]
            Name of this hook.

        script : typing.Optional[str]
            Code to be executed when this hook runs.

        enabled : typing.Optional[bool]
            Whether this hook will be executed (true) or ignored (false).

        dependencies : typing.Optional[HookDependencies]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateHookResponseContent
            Hook successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.hooks.update(
            id="id",
        )
        """
        _response = self._raw_client.update(
            id, name=name, script=script, enabled=enabled, dependencies=dependencies, request_options=request_options
        )
        return _response.data

    @property
    def secrets(self):
        if self._secrets is None:
            from .secrets.client import SecretsClient  # noqa: E402

            self._secrets = SecretsClient(client_wrapper=self._client_wrapper)
        return self._secrets


class AsyncHooksClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawHooksClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._secrets: typing.Optional[AsyncSecretsClient] = None

    @property
    def with_raw_response(self) -> AsyncRawHooksClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawHooksClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        enabled: typing.Optional[bool] = None,
        fields: typing.Optional[str] = None,
        trigger_id: typing.Optional[HookTriggerIdEnum] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Hook, ListHooksOffsetPaginatedResponseContent]:
        """
        Retrieve all [hooks](https://auth0.com/docs/hooks). Accepts a list of fields to include or exclude in the result.

        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        enabled : typing.Optional[bool]
            Optional filter on whether a hook is enabled (true) or disabled (false).

        fields : typing.Optional[str]
            Comma-separated list of fields to include in the result. Leave empty to retrieve all fields.

        trigger_id : typing.Optional[HookTriggerIdEnum]
            Retrieves hooks that match the trigger

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Hook, ListHooksOffsetPaginatedResponseContent]
            Hooks successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.hooks.list(
                page=1,
                per_page=1,
                include_totals=True,
                enabled=True,
                fields="fields",
                trigger_id="credentials-exchange",
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            page=page,
            per_page=per_page,
            include_totals=include_totals,
            enabled=enabled,
            fields=fields,
            trigger_id=trigger_id,
            request_options=request_options,
        )

    async def create(
        self,
        *,
        name: str,
        script: str,
        trigger_id: HookTriggerIdEnum,
        enabled: typing.Optional[bool] = OMIT,
        dependencies: typing.Optional[HookDependencies] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateHookResponseContent:
        """
        Create a new hook.

        Parameters
        ----------
        name : str
            Name of this hook.

        script : str
            Code to be executed when this hook runs.

        trigger_id : HookTriggerIdEnum
            Execution stage of this rule. Can be `credentials-exchange`, `pre-user-registration`, `post-user-registration`, `post-change-password`, or `send-phone-message`.

        enabled : typing.Optional[bool]
            Whether this hook will be executed (true) or ignored (false).

        dependencies : typing.Optional[HookDependencies]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateHookResponseContent
            Hook successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.hooks.create(
                name="name",
                script="script",
                trigger_id="credentials-exchange",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name,
            script=script,
            trigger_id=trigger_id,
            enabled=enabled,
            dependencies=dependencies,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, id: str, *, fields: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> GetHookResponseContent:
        """
        Retrieve [a hook](https://auth0.com/docs/hooks) by its ID. Accepts a list of fields to include in the result.

        Parameters
        ----------
        id : str
            ID of the hook to retrieve.

        fields : typing.Optional[str]
            Comma-separated list of fields to include in the result. Leave empty to retrieve all fields.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetHookResponseContent
            Hook successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.hooks.get(
                id="id",
                fields="fields",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, fields=fields, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a hook.

        Parameters
        ----------
        id : str
            ID of the hook to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.hooks.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        script: typing.Optional[str] = OMIT,
        enabled: typing.Optional[bool] = OMIT,
        dependencies: typing.Optional[HookDependencies] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateHookResponseContent:
        """
        Update an existing hook.

        Parameters
        ----------
        id : str
            ID of the hook to update.

        name : typing.Optional[str]
            Name of this hook.

        script : typing.Optional[str]
            Code to be executed when this hook runs.

        enabled : typing.Optional[bool]
            Whether this hook will be executed (true) or ignored (false).

        dependencies : typing.Optional[HookDependencies]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateHookResponseContent
            Hook successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.hooks.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id, name=name, script=script, enabled=enabled, dependencies=dependencies, request_options=request_options
        )
        return _response.data

    @property
    def secrets(self):
        if self._secrets is None:
            from .secrets.client import AsyncSecretsClient  # noqa: E402

            self._secrets = AsyncSecretsClient(client_wrapper=self._client_wrapper)
        return self._secrets



from .conftest import get_client, verify_request_count


def test_rules_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "rules.list_.0"
    client = get_client(test_id)
    client.rules.list(
        page=1,
        per_page=1,
        include_totals=True,
        enabled=True,
        fields="fields",
        include_fields=True,
    )
    verify_request_count(
        test_id,
        "GET",
        "/rules",
        {
            "page": "1",
            "per_page": "1",
            "include_totals": "true",
            "enabled": "true",
            "fields": "fields",
            "include_fields": "true",
        },
        1,
    )


def test_rules_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "rules.create.0"
    client = get_client(test_id)
    client.rules.create(
        name="name",
        script="script",
    )
    verify_request_count(test_id, "POST", "/rules", None, 1)


def test_rules_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "rules.get.0"
    client = get_client(test_id)
    client.rules.get(
        id="id",
        fields="fields",
        include_fields=True,
    )
    verify_request_count(test_id, "GET", "/rules/id", {"fields": "fields", "include_fields": "true"}, 1)


def test_rules_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "rules.delete.0"
    client = get_client(test_id)
    client.rules.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/rules/id", None, 1)


def test_rules_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "rules.update.0"
    client = get_client(test_id)
    client.rules.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/rules/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .csp_directives import CspDirectives
from .csp_flags import CspFlags
from .csp_policy_mode import CspPolicyMode
from .csp_policy_reporting import CspPolicyReporting


class CspPolicy(UniversalBaseModel):
    """
    A single CSP policy with mode, directives, flags, and optional reporting.
    """

    mode: typing.Optional[CspPolicyMode] = None
    directives: typing.Optional[CspDirectives] = None
    flags: typing.Optional[CspFlags] = None
    reporting: typing.Optional[CspPolicyReporting] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .branding_theme_colors_captcha_widget_theme_enum import BrandingThemeColorsCaptchaWidgetThemeEnum


class BrandingThemeColors(UniversalBaseModel):
    base_focus_color: typing.Optional[str] = pydantic.Field(default=None)
    """
    Base Focus Color
    """

    base_hover_color: typing.Optional[str] = pydantic.Field(default=None)
    """
    Base Hover Color
    """

    body_text: str = pydantic.Field()
    """
    Body text
    """

    captcha_widget_theme: typing.Optional[BrandingThemeColorsCaptchaWidgetThemeEnum] = None
    error: str = pydantic.Field()
    """
    Error
    """

    header: str = pydantic.Field()
    """
    Header
    """

    icons: str = pydantic.Field()
    """
    Icons
    """

    input_background: str = pydantic.Field()
    """
    Input background
    """

    input_border: str = pydantic.Field()
    """
    Input border
    """

    input_filled_text: str = pydantic.Field()
    """
    Input filled text
    """

    input_labels_placeholders: str = pydantic.Field()
    """
    Input labels & placeholders
    """

    links_focused_components: str = pydantic.Field()
    """
    Links & focused components
    """

    primary_button: str = pydantic.Field()
    """
    Primary button
    """

    primary_button_label: str = pydantic.Field()
    """
    Primary button label
    """

    read_only_background: typing.Optional[str] = pydantic.Field(default=None)
    """
    Read only background
    """

    secondary_button_border: str = pydantic.Field()
    """
    Secondary button border
    """

    secondary_button_label: str = pydantic.Field()
    """
    Secondary button label
    """

    success: str = pydantic.Field()
    """
    Success
    """

    widget_background: str = pydantic.Field()
    """
    Widget background
    """

    widget_border: str = pydantic.Field()
    """
    Widget border
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .connection_options_client_id_twitter import ConnectionOptionsClientIdTwitter
from .connection_options_client_secret_twitter import ConnectionOptionsClientSecretTwitter
from .connection_options_common import ConnectionOptionsCommon
from .connection_options_protocol_enum_twitter import ConnectionOptionsProtocolEnumTwitter
from .connection_options_scope_twitter import ConnectionOptionsScopeTwitter
from .connection_scope_array import ConnectionScopeArray
from .connection_set_user_root_attributes_enum import ConnectionSetUserRootAttributesEnum
from .connection_upstream_params import ConnectionUpstreamParams


class ConnectionOptionsTwitter(ConnectionOptionsCommon):
    """
    Options for the 'twitter' connection
    """

    client_id: typing.Optional[ConnectionOptionsClientIdTwitter] = None
    client_secret: typing.Optional[ConnectionOptionsClientSecretTwitter] = None
    freeform_scopes: typing.Optional[ConnectionScopeArray] = None
    protocol: typing.Optional[ConnectionOptionsProtocolEnumTwitter] = None
    scope: typing.Optional[ConnectionOptionsScopeTwitter] = None
    set_user_root_attributes: typing.Optional[ConnectionSetUserRootAttributesEnum] = None
    upstream_params: typing.Optional[ConnectionUpstreamParams] = None
    offline_access: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Request long-lived refresh tokens so your app can act on behalf of users even when they’re not actively signed in. Typical Twitter use case: keeping a background service synced without forcing users to reauthorize every session.
    """

    profile: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Pull account profile metadata such as display name, bio, location, and URL so downstream apps can prefill or personalize user experiences.
    """

    tweet_read: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Allow the application to read a user’s public and protected Tweets—required for timelines, analytics, or moderation workflows.
    """

    users_read: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Read non-Tweet user information (e.g., followers/following, account settings) to power relationship graphs or audience insights.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_connections_users_delete_by_email() -> None:
    """Test deleteByEmail endpoint with WireMock"""
    test_id = "connections.users.delete_by_email.0"
    client = get_client(test_id)
    client.connections.users.delete_by_email(
        id="id",
        email="email",
    )
    verify_request_count(test_id, "DELETE", "/connections/id/users", {"email": "email"}, 1)



from .conftest import get_client, verify_request_count

from auth0.management import PatchRateLimitPolicyConfigurationRequestContentZero, RateLimitPolicyConfigurationZero


def test_rateLimitPolicies_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "rate_limit_policies.list_.0"
    client = get_client(test_id)
    client.rate_limit_policies.list(
        resource="oauth_authentication_api",
        consumer="client",
        consumer_selector="consumer_selector",
        take=1,
        from_="from",
    )
    verify_request_count(
        test_id,
        "GET",
        "/rate-limit-policies",
        {
            "resource": "oauth_authentication_api",
            "consumer": "client",
            "consumer_selector": "consumer_selector",
            "take": "1",
            "from": "from",
        },
        1,
    )


def test_rateLimitPolicies_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "rate_limit_policies.create.0"
    client = get_client(test_id)
    client.rate_limit_policies.create(
        resource="oauth_authentication_api",
        consumer="client",
        consumer_selector="consumer_selector",
        configuration=RateLimitPolicyConfigurationZero(
            action="allow",
        ),
    )
    verify_request_count(test_id, "POST", "/rate-limit-policies", None, 1)


def test_rateLimitPolicies_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "rate_limit_policies.get.0"
    client = get_client(test_id)
    client.rate_limit_policies.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/rate-limit-policies/id", None, 1)


def test_rateLimitPolicies_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "rate_limit_policies.delete.0"
    client = get_client(test_id)
    client.rate_limit_policies.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/rate-limit-policies/id", None, 1)


def test_rateLimitPolicies_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "rate_limit_policies.update.0"
    client = get_client(test_id)
    client.rate_limit_policies.update(
        id="id",
        configuration=PatchRateLimitPolicyConfigurationRequestContentZero(
            action="allow",
        ),
    )
    verify_request_count(test_id, "PATCH", "/rate-limit-policies/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

ClientMyOrganizationConfigurationAllowedStrategiesEnum = typing.Union[
    typing.Literal["pingfederate", "adfs", "waad", "google-apps", "okta", "oidc", "samlp"], typing.Any
]



from .conftest import get_client, verify_request_count


def test_guardian_enrollments_create_ticket() -> None:
    """Test createTicket endpoint with WireMock"""
    test_id = "guardian.enrollments.create_ticket.0"
    client = get_client(test_id)
    client.guardian.enrollments.create_ticket(
        user_id="user_id",
    )
    verify_request_count(test_id, "POST", "/guardian/enrollments/ticket", None, 1)


def test_guardian_enrollments_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "guardian.enrollments.get.0"
    client = get_client(test_id)
    client.guardian.enrollments.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/guardian/enrollments/id", None, 1)


def test_guardian_enrollments_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "guardian.enrollments.delete.0"
    client = get_client(test_id)
    client.guardian.enrollments.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/guardian/enrollments/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .phone_template_body import PhoneTemplateBody


class PhoneTemplateContent(UniversalBaseModel):
    syntax: typing.Optional[str] = None
    from_: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="from"),
        pydantic.Field(
            alias="from", description="Default phone number to be used as 'from' when sending a phone notification"
        ),
    ] = None
    """
    Default phone number to be used as 'from' when sending a phone notification
    """

    body: typing.Optional[PhoneTemplateBody] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_users_organizations_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "users.organizations.list_.0"
    client = get_client(test_id)
    client.users.organizations.list(
        id="id",
        page=1,
        per_page=1,
        include_totals=True,
    )
    verify_request_count(
        test_id, "GET", "/users/id/organizations", {"page": "1", "per_page": "1", "include_totals": "true"}, 1
    )



from .conftest import get_client, verify_request_count


def test_sessions_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "sessions.get.0"
    client = get_client(test_id)
    client.sessions.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/sessions/id", None, 1)


def test_sessions_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "sessions.delete.0"
    client = get_client(test_id)
    client.sessions.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/sessions/id", None, 1)


def test_sessions_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "sessions.update.0"
    client = get_client(test_id)
    client.sessions.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/sessions/id", None, 1)


def test_sessions_revoke() -> None:
    """Test revoke endpoint with WireMock"""
    test_id = "sessions.revoke.0"
    client = get_client(test_id)
    client.sessions.revoke(
        id="id",
    )
    verify_request_count(test_id, "POST", "/sessions/id/revoke", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.list_self_service_profile_custom_text_response_content import (
    ListSelfServiceProfileCustomTextResponseContent,
)
from ...types.self_service_profile_custom_text_language_enum import SelfServiceProfileCustomTextLanguageEnum
from ...types.self_service_profile_custom_text_page_enum import SelfServiceProfileCustomTextPageEnum
from ...types.set_self_service_profile_custom_text_request_content import SetSelfServiceProfileCustomTextRequestContent
from ...types.set_self_service_profile_custom_text_response_content import (
    SetSelfServiceProfileCustomTextResponseContent,
)
from .raw_client import AsyncRawCustomTextClient, RawCustomTextClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CustomTextClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCustomTextClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCustomTextClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCustomTextClient
        """
        return self._raw_client

    def list(
        self,
        id: str,
        language: SelfServiceProfileCustomTextLanguageEnum,
        page: SelfServiceProfileCustomTextPageEnum,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListSelfServiceProfileCustomTextResponseContent:
        """
        Retrieves text customizations for a given self-service profile, language and Self-Service Enterprise Configuration flow page.

        Parameters
        ----------
        id : str
            The id of the self-service profile.

        language : SelfServiceProfileCustomTextLanguageEnum
            The language of the custom text.

        page : SelfServiceProfileCustomTextPageEnum
            The page where the custom text is shown.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListSelfServiceProfileCustomTextResponseContent
            Retrieved custom text.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.self_service_profiles.custom_text.list(
            id="id",
            language="en",
            page="get-started",
        )
        """
        _response = self._raw_client.list(id, language, page, request_options=request_options)
        return _response.data

    def set(
        self,
        id: str,
        language: SelfServiceProfileCustomTextLanguageEnum,
        page: SelfServiceProfileCustomTextPageEnum,
        *,
        request: SetSelfServiceProfileCustomTextRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SetSelfServiceProfileCustomTextResponseContent:
        """
        Updates text customizations for a given self-service profile, language and Self-Service Enterprise Configuration flow page.

        Parameters
        ----------
        id : str
            The id of the self-service profile.

        language : SelfServiceProfileCustomTextLanguageEnum
            The language of the custom text.

        page : SelfServiceProfileCustomTextPageEnum
            The page where the custom text is shown.

        request : SetSelfServiceProfileCustomTextRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SetSelfServiceProfileCustomTextResponseContent
            Updated custom text.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.self_service_profiles.custom_text.set(
            id="id",
            language="en",
            page="get-started",
            request={"key": "value"},
        )
        """
        _response = self._raw_client.set(id, language, page, request=request, request_options=request_options)
        return _response.data


class AsyncCustomTextClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCustomTextClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCustomTextClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCustomTextClient
        """
        return self._raw_client

    async def list(
        self,
        id: str,
        language: SelfServiceProfileCustomTextLanguageEnum,
        page: SelfServiceProfileCustomTextPageEnum,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListSelfServiceProfileCustomTextResponseContent:
        """
        Retrieves text customizations for a given self-service profile, language and Self-Service Enterprise Configuration flow page.

        Parameters
        ----------
        id : str
            The id of the self-service profile.

        language : SelfServiceProfileCustomTextLanguageEnum
            The language of the custom text.

        page : SelfServiceProfileCustomTextPageEnum
            The page where the custom text is shown.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListSelfServiceProfileCustomTextResponseContent
            Retrieved custom text.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.self_service_profiles.custom_text.list(
                id="id",
                language="en",
                page="get-started",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(id, language, page, request_options=request_options)
        return _response.data

    async def set(
        self,
        id: str,
        language: SelfServiceProfileCustomTextLanguageEnum,
        page: SelfServiceProfileCustomTextPageEnum,
        *,
        request: SetSelfServiceProfileCustomTextRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SetSelfServiceProfileCustomTextResponseContent:
        """
        Updates text customizations for a given self-service profile, language and Self-Service Enterprise Configuration flow page.

        Parameters
        ----------
        id : str
            The id of the self-service profile.

        language : SelfServiceProfileCustomTextLanguageEnum
            The language of the custom text.

        page : SelfServiceProfileCustomTextPageEnum
            The page where the custom text is shown.

        request : SetSelfServiceProfileCustomTextRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SetSelfServiceProfileCustomTextResponseContent
            Updated custom text.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.self_service_profiles.custom_text.set(
                id="id",
                language="en",
                page="get-started",
                request={"key": "value"},
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.set(id, language, page, request=request, request_options=request_options)
        return _response.data
